import os
from typing import Union
import yaml
import pathlib
import colorama
import sys


class ConfigurationLoader:
    PURPLE = colorama.Fore.MAGENTA
    WHITE = colorama.Style.RESET_ALL
    RED = colorama.Fore.RED

    @staticmethod
    def load(path):
        """
        Loads Saveddit configuration from a configuration file.
        If ifle is not found, create one and exit.

        Arguments:
            path: path to user_config.yaml file

        Returns:
            A Python dictionary with Saveddit configuration info
        """

        def _create_config(_path):
            _STD_CONFIG = {
                "reddit_client_id": "",
                "reddit_client_secret": "",
                "reddit_username": "",
                "imgur_client_id": "",
            }
            with open(_path, "x") as _f:
                yaml.dump(_STD_CONFIG, _f)
            sys.exit(0)

        # Explicitly converting path to POSIX-like path (to avoid '\\' hell)
        print(
            "{notice}Retrieving configuration from {path} file{white}".format(
                path=path,
                notice=ConfigurationLoader.PURPLE,
                white=ConfigurationLoader.WHITE,
            )
        )
        path = pathlib.Path(path).absolute().as_posix()

        # Check if file exists. If not, create one and fill it with std config template
        if not os.path.exists(path):
            print(
                "{red}No configuration file found.\nCreating one. Please edit {path} with valid credentials.\nExiting{white}".format(
                    red=ConfigurationLoader.RED,
                    path=path,
                    white=ConfigurationLoader.WHITE,
                )
            )
            _create_config(path)

        with open(path, "r") as _f:
            return yaml.safe_load(_f.read())



import coloredlogs
from colorama import Fore
import logging
import verboselogs
import os
import praw
from saveddit.configuration import ConfigurationLoader
from saveddit.submission_downloader import SubmissionDownloader

class SubredditDownloader:
    app_config_dir = os.path.expanduser("~/.saveddit")
    if not os.path.exists(app_config_dir):
        os.makedirs(app_config_dir)

    config_file_location = os.path.expanduser("~/.saveddit/user_config.yaml")
    config = ConfigurationLoader.load(config_file_location)

    REDDIT_CLIENT_ID = config['reddit_client_id']
    REDDIT_CLIENT_SECRET = config['reddit_client_secret']
    IMGUR_CLIENT_ID = config['imgur_client_id']
    DEFAULT_CATEGORIES = ["hot", "new", "rising",
                          "controversial", "top", "gilded"]
    DEFAULT_POST_LIMIT = None

    def __init__(self, subreddit_name):
        self.subreddit_name = subreddit_name
        reddit = praw.Reddit(
            client_id=SubredditDownloader.REDDIT_CLIENT_ID,
            client_secret=SubredditDownloader.REDDIT_CLIENT_SECRET,
            user_agent="saveddit (by /u/p_ranav)",
        )
        self.subreddit = reddit.subreddit(subreddit_name)

        self.logger = verboselogs.VerboseLogger(__name__)
        level_styles = {
            'critical': {'bold': True, 'color': 'red'},
            'debug': {'color': 'green'},
            'error': {'color': 'red'},
            'info': {'color': 'white'},
            'notice': {'color': 'magenta'},
            'spam': {'color': 'white', 'faint': True},
            'success': {'bold': True, 'color': 'green'},
            'verbose': {'color': 'blue'},
            'warning': {'color': 'yellow'}
        }
        coloredlogs.install(level='SPAM', logger=self.logger,
                            fmt='%(message)s', level_styles=level_styles)

    def download(self, output_path, categories=DEFAULT_CATEGORIES, post_limit=DEFAULT_POST_LIMIT, skip_videos=False, skip_meta=False, skip_comments=False, comment_limit=0):
        '''
        categories: List of categories within the subreddit to download (see SubredditDownloader.DEFAULT_CATEGORIES)
        post_limit: Number of posts to download (default: None, i.e., all posts)
        comment_limit: Number of comment levels to download from submission (default: `0`, i.e., only top-level comments)
          - to get all comments, set comment_limit to `None`
        '''
        root_dir = os.path.join(os.path.join(os.path.join(
            output_path, "www.reddit.com"), "r"), self.subreddit_name)
        categories = categories

        for c in categories:
            self.logger.notice("Downloading from /r/" +
                               self.subreddit_name + "/" + c + "/")
            category_dir = os.path.join(root_dir, c)
            if not os.path.exists(category_dir):
                os.makedirs(category_dir)
            category_function = getattr(self.subreddit, c)

            for i, submission in enumerate(category_function(limit=post_limit)):
                SubmissionDownloader(submission, i, self.logger, category_dir,
                    skip_videos, skip_meta, skip_comments, comment_limit,
                    {'imgur_client_id': SubredditDownloader.IMGUR_CLIENT_ID})



from bs4 import BeautifulSoup
import coloredlogs
from colorama import Fore
import contextlib
import logging
import verboselogs
from datetime import datetime
import os
from io import StringIO
import json
import ffmpeg
import praw
from pprint import pprint
import re
import requests
from tqdm import tqdm
import urllib.request
import youtube_dl


class SubmissionDownloader:
    def __init__(self, submission, submission_index, logger, output_dir, skip_videos, skip_meta, skip_comments, comment_limit, config):
        self.IMGUR_CLIENT_ID = config["imgur_client_id"]

        self.logger = logger
        i = submission_index
        prefix_str = '#' + str(i).zfill(3) + ' '
        self.indent_1 = ' ' * len(prefix_str) + "* "
        self.indent_2 = ' ' * len(self.indent_1) + "- "

        has_url = getattr(submission, "url", None)
        if has_url:
            title = submission.title
            self.logger.verbose(prefix_str + '"' + title + '"')
            title = re.sub(r'\W+', '_', title)

            # Truncate title
            if len(title) > 32:
                title = title[0:32]
                title += "..."

            # Prepare directory for the submission
            post_dir = str(i).zfill(3) + "_" + title.replace(" ", "_")
            submission_dir = os.path.join(output_dir, post_dir)
            if not os.path.exists(submission_dir):
                os.makedirs(submission_dir)

            self.logger.spam(
                self.indent_1 + "Processing `" + submission.url + "`")

            success = False

            should_create_files_dir = True
            if skip_comments and skip_meta:
                should_create_files_dir = False

            def create_files_dir(submission_dir):
                if should_create_files_dir:
                    files_dir = os.path.join(submission_dir, "files")
                    if not os.path.exists(files_dir):
                        os.makedirs(files_dir)
                    return files_dir
                else:
                    return submission_dir

            if self.is_direct_link_to_content(submission.url, [".png", ".jpg", ".jpeg", ".gif"]):
                files_dir = create_files_dir(submission_dir)

                filename = submission.url.split("/")[-1]
                self.logger.spam(
                    self.indent_1 + "This is a direct link to a " + filename.split(".")[-1] + " file")
                save_path = os.path.join(files_dir, filename)
                self.download_direct_link(submission, save_path)
                success = True
            elif self.is_direct_link_to_content(submission.url, [".mp4"]):
                self.logger.spam(
                    self.indent_1 + "This is a direct link to a " + filename.split(".")[-1] + " file")
                if not skip_videos:
                    files_dir = create_files_dir(submission_dir)

                    filename = submission.url.split("/")[-1]
                    save_path = os.path.join(files_dir, filename)
                    self.download_direct_link(submission, save_path)
                    success = True
                else:
                    self.logger.spam(self.indent_1 + "Skipping download of video content")
                    success = True
            elif self.is_reddit_gallery(submission.url):
                files_dir = create_files_dir(submission_dir)

                self.logger.spam(
                    self.indent_1 + "This is a reddit gallery")
                self.download_reddit_gallery(submission, files_dir, skip_videos)
                success = True
            elif self.is_reddit_video(submission.url):
                self.logger.spam(
                    self.indent_1 + "This is a reddit video")

                if not skip_videos:
                    files_dir = create_files_dir(submission_dir)
                    self.download_reddit_video(submission, files_dir)
                    success = True
                else:
                    self.logger.spam(self.indent_1 + "Skipping download of video content")
                    success = True
            elif self.is_gfycat_link(submission.url) or self.is_redgifs_link(submission.url):
                if self.is_gfycat_link(submission.url):
                    self.logger.spam(
                        self.indent_1 + "This is a gfycat link")
                else:
                    self.logger.spam(
                        self.indent_1 + "This is a redgif link")

                if not skip_videos:
                    files_dir = create_files_dir(submission_dir)
                    self.download_gfycat_or_redgif(submission, files_dir)
                    success = True
                else:
                    self.logger.spam(self.indent_1 + "Skipping download of video content")
                    success = True
            elif self.is_imgur_album(submission.url):
                files_dir = create_files_dir(submission_dir)

                self.logger.spam(
                    self.indent_1 + "This is an imgur album")
                self.download_imgur_album(submission, files_dir)
                success = True
            elif self.is_imgur_image(submission.url):
                files_dir = create_files_dir(submission_dir)

                self.logger.spam(
                    self.indent_1 + "This is an imgur image or video")
                self.download_imgur_image(submission, files_dir)
                success = True
            elif self.is_self_post(submission):
                self.logger.spam(self.indent_1 + "This is a self-post")
                success = True
            elif (not skip_videos) and (self.is_youtube_link(submission.url) or self.is_supported_by_youtubedl(submission.url)):
                if self.is_youtube_link(submission.url):
                    self.logger.spam(
                        self.indent_1 + "This is a youtube link")
                else:
                    self.logger.spam(
                        self.indent_1 + "This link is supported by a youtube-dl extractor")

                if not skip_videos:
                    files_dir = create_files_dir(submission_dir)
                    self.download_youtube_video(submission.url, files_dir)
                    success = True
                else:
                    self.logger.spam(self.indent_1 + "Skipping download of video content")
                    success = True
            else:
                success = True

            # Download submission meta
            if not skip_meta:
                self.logger.spam(self.indent_1 + "Saving submission.json")
                self.download_submission_meta(submission, submission_dir)
            else:
                self.logger.spam(
                    self.indent_1 + "Skipping submissions meta")

            # Downlaod comments if requested
            if not skip_comments:
                if comment_limit == None:
                    self.logger.spam(
                        self.indent_1 + "Saving all comments to comments.json")
                else:
                    self.logger.spam(
                        self.indent_1 + "Saving top-level comments to comments.json")
                self.download_comments(
                    submission, submission_dir, comment_limit)
            else:
                self.logger.spam(
                    self.indent_1 + "Skipping comments")

            if success:
                self.logger.spam(
                    self.indent_1 + "Saved to " + submission_dir + "\n")
            else:
                self.logger.warning(
                    self.indent_1 + "Failed to download from link " + submission.url + "\n"
                )

    def print_formatted_error(self, e):
        for line in str(e).split("\n"):
            self.logger.error(self.indent_2 + line)

    def is_direct_link_to_content(self, url, supported_file_formats):
        url_leaf = url.split("/")[-1]
        return any([i in url_leaf for i in supported_file_formats]) and ".gifv" not in url_leaf

    def download_direct_link(self, submission, output_path):
        try:
            urllib.request.urlretrieve(submission.url, output_path)
        except Exception as e:
            self.logger.error(e)

    def is_youtube_link(self, url):
        return "youtube.com" in url or "youtu.be" in url

    def is_supported_by_youtubedl(self, url):
        try:
            # Since youtube-dl's quiet mode is amything BUT quiet
            # using contextlib to redirect stdout to a local StringIO variable
            local_stderr = StringIO()
            with contextlib.redirect_stderr(local_stderr):
                if "flickr.com/photos" in url:
                    return False

                # Try to extract info from URL
                try:
                    download_options = {
                        'quiet': True,
                        'warnings': True,
                        'ignoreerrors': True,
                    }
                    ydl = youtube_dl.YoutubeDL(download_options)
                    r = ydl.extract_info(url, download=False)
                except Exception as e:
                    # No media found through youtube-dl
                    self.logger.spam(self.indent_2 + "No media found in '" + url + "' that could be downloaded with youtube-dl")
                    return False

                extractors = youtube_dl.extractor.gen_extractors()
                for e in extractors:
                    if e.suitable(url) and e.IE_NAME != 'generic':
                        return True
                        self.logger.spam(self.indent_2 + "This link could potentially be downloaded with youtube-dl")
                self.logger.spam(self.indent_2 + "No media found in '" + url + "' that could be downloaded with youtube-dl")
                return False
        except Exception as e:
            return False

    def download_youtube_video(self, url, output_path):
        try:
            local_stderr = StringIO()
            with contextlib.redirect_stderr(local_stderr):
                download_options = {
                    'format': "299+bestaudio/298+bestaudio/137+bestaudio/136+bestaudio/best",
                    'quiet': True,
                    'warnings': True,
                    'ignoreerrors': True,
                    'nooverwrites': True,
                    'continuedl': True,
                    'outtmpl': output_path + '/%(id)s.%(ext)s'
                }
                self.logger.spam(self.indent_2 + "Downloading " +
                                url + " with youtube-dl")
                with youtube_dl.YoutubeDL(download_options) as ydl:
                    ydl.download([url])
                    errors = local_stderr.getvalue()
                    if not len(errors):
                        self.logger.spam(self.indent_2 + "Finished downloading video from " +
                                    url)
                    else:
                        self.logger.error(self.indent_2 + errors.strip())
        except Exception as e:
            self.logger.error(self.indent_2 + "Failed to download with youtube-dl")
            self.print_formatted_error(e)

    def is_reddit_gallery(self, url):
        return "reddit.com/gallery" in url

    def download_reddit_gallery(self, submission, output_path, skip_videos):
        gallery_data = getattr(submission, "gallery_data", None)
        media_metadata = getattr(submission, "media_metadata", None)
        self.logger.spam(
            self.indent_2 + "Looking for submission.gallery_data and submission.media_metadata")

        if gallery_data == None and media_metadata == None:
            # gallery_data not in submission
            # could be a crosspost
            crosspost_parent_list = getattr(
                submission, "crosspost_parent_list", None)
            if crosspost_parent_list != None:
                self.logger.spam(
                    self.indent_2 + "This is a crosspost to a reddit gallery")
                first_parent = crosspost_parent_list[0]
                gallery_data = first_parent["gallery_data"]
                media_metadata = first_parent["media_metadata"]

        if gallery_data != None and media_metadata != None:
            image_count = len(gallery_data["items"])
            self.logger.spam(self.indent_2 + "This reddit gallery has " +
                             str(image_count) + " images")
            for j, item in tqdm(enumerate(gallery_data["items"]), total=image_count, bar_format='%s%s{l_bar}{bar:20}{r_bar}%s' % (self.indent_2, Fore.WHITE + Fore.LIGHTBLACK_EX, Fore.RESET)):
                media_id = item["media_id"]
                item_metadata = media_metadata[media_id]
                item_format = item_metadata['m']
                if "image/" in item_format or "video/" in item_format:
                    if not os.path.exists(output_path):
                        os.makedirs(output_path)
                    if "image/" in item_format:
                        item_format = item_format.split("image/")[-1]
                    elif "video/" in item_format:
                        item_format = item_format.split("video/")[-1]
                        # Skip video content if requested by user
                        if skip_videos:
                            continue
                    item_filename = media_id + "." + item_format
                    item_url = item_metadata["s"]["u"]
                    save_path = os.path.join(output_path, item_filename)
                    try:
                        urllib.request.urlretrieve(item_url, save_path)
                    except Exception as e:
                        self.print_formatted_error(e)

    def is_reddit_video(self, url):
        return "v.redd.it" in url

    def download_reddit_video(self, submission, output_path):
        media = getattr(submission, "media", None)
        media_id = submission.url.split("v.redd.it/")[-1]

        self.logger.spam(self.indent_2 + "Looking for submission.media")

        if media == None:
            # link might be a crosspost
            crosspost_parent_list = getattr(
                submission, "crosspost_parent_list", None)
            if crosspost_parent_list != None:
                self.logger.spam(
                    self.indent_2 + "This is a crosspost to a reddit video")
                first_parent = crosspost_parent_list[0]
                media = first_parent["media"]

        if media != None:
            self.logger.spam(self.indent_2 + "Downloading video component")
            url = media["reddit_video"]["fallback_url"]
            video_save_path = os.path.join(
                output_path, media_id + "_video.mp4")
            try:
                urllib.request.urlretrieve(url, video_save_path)
            except Exception as e:
                self.print_formatted_error(e)

            # Download the audio
            self.logger.spam(self.indent_2 + "Downloading audio component")
            audio_downloaded = False
            audio_save_path = os.path.join(
                output_path, media_id + "_audio.mp4")
            try:
                urllib.request.urlretrieve(
                    submission.url + "/DASH_audio.mp4", audio_save_path)
                audio_downloaded = True
            except Exception as e:
                pass

            if audio_downloaded == True:
                # Merge mp4 files
                self.logger.spam(
                    self.indent_2 + "Merging video & audio components with ffmpeg")
                output_save_path = os.path.join(output_path, media_id + ".mp4")
                input_video = ffmpeg.input(video_save_path)
                input_audio = ffmpeg.input(audio_save_path)
                ffmpeg.concat(input_video, input_audio, v=1, a=1)\
                    .output(output_save_path)\
                    .global_args('-loglevel', 'error')\
                      .global_args('-y')\
                    .run()
                self.logger.spam(self.indent_2 + "Done merging with ffmpeg")
            else:
                self.logger.spam(
                    self.indent_2 + "This video does not have an audio component")

            self.logger.spam(
                    self.indent_2 + "Sucessfully saved video")

    def is_gfycat_link(self, url):
        return "gfycat.com/" in url

    def is_redgifs_link(self, url):
        return "redgifs.com/" in url

    def get_gfycat_embedded_video_url(self, url):
        try:
            response  = requests.get(url)
            data = response.text
            soup = BeautifulSoup(data, 'lxml')

            # Cycle through all links
            for link in soup.find_all():
                link_src = link.get('src')
                src_url = str(link_src)
                if ".mp4" in src_url:
                    # Looking for giant.gfycat.com
                    if "giant." in src_url:
                        return src_url
        except Exception as e:
            return ""

    def download_gfycat_or_redgif(self, submission, output_dir):
        self.logger.spam(
            self.indent_2 + "Looking for submission.preview.reddit_video_preview.fallback_url")
        preview = None
        try:
            preview = getattr(submission, "preview")
            if preview:
                if "reddit_video_preview" in preview:
                    if "fallback_url" in preview["reddit_video_preview"]:
                        fallback_url = preview["reddit_video_preview"]["fallback_url"]
                        file_format = fallback_url.split(".")[-1]
                        filename = submission.url.split("/")[-1] + "." + file_format
                        save_path = os.path.join(output_dir, filename)
                        try:
                            urllib.request.urlretrieve(fallback_url, save_path)
                            return
                        except Exception as e:
                            self.print_formatted_error(e)
        except Exception as e:
           self.print_formatted_error(e)

        try:
            self.logger.spam(
                self.indent_2 + "Looking for submission.media_embed")
            media_embed = getattr(submission, "media_embed")
            if media_embed:
                content = media_embed["content"]
                self.logger.spam(
                    self.indent_2 + "Found submission.media_embed")
                if "iframe" in content:
                    if "gfycat.com" in submission.url:
                        self.logger.spam(
                            self.indent_2 + "This is an embedded video in gfycat.com")
                        # This is likely an embedded video in gfycat
                        video_url = self.get_gfycat_embedded_video_url(submission.url)
                        if len(video_url):
                            filename = video_url.split("/")[-1]
                            save_path = os.path.join(output_dir, filename)

                            self.logger.spam(
                                self.indent_2 + "Embedded video URL: " + video_url)
                            try:
                                urllib.request.urlretrieve(video_url, save_path)
                                return
                            except Exception as e:
                                self.print_formatted_error(e)
        except Exception as e:
            self.print_formatted_error(e)

    def is_imgur_album(self, url):
        return "imgur.com/a/" in url or "imgur.com/gallery/" in url

    def get_imgur_album_images_count(self, album_id):
        request = "https://api.imgur.com/3/album/" + album_id
        res = requests.get(request, headers={
                           "Authorization": "Client-ID " + self.IMGUR_CLIENT_ID})
        if res.status_code == 200:
            return res.json()["data"]["images_count"]
        else:
            self.logger.spam(self.indent_2 + "This imgur album is empty")
            return 0

    def get_imgur_image_meta(self, image_id):
        request = "https://api.imgur.com/3/image/" + image_id
        res = requests.get(request, headers={
                           "Authorization": "Client-ID " + self.IMGUR_CLIENT_ID})
        return res.json()["data"]

    def download_imgur_album(self, submission, output_dir):
        # Imgur album
        album_id = ""
        if "imgur.com/a/" in submission.url:
            album_id = submission.url.split("imgur.com/a/")[-1]
        elif "imgur.com/gallery/" in submission.url:
            album_id = submission.url.split("imgur.com/gallery/")[-1]

        self.logger.spam(self.indent_2 + "Album ID " + album_id)

        images_count = self.get_imgur_album_images_count(album_id)
        if images_count > 0:
            request = "https://api.imgur.com/3/album/" + album_id
            res = requests.get(request, headers={
                               "Authorization": "Client-ID " + self.IMGUR_CLIENT_ID})
            self.logger.spam(self.indent_2 + "This imgur album has " +
                             str(images_count) + " images")
            for i, image in tqdm(enumerate(res.json()["data"]["images"]), total=images_count, bar_format='%s%s{l_bar}{bar:20}{r_bar}%s' % (self.indent_2, Fore.WHITE + Fore.LIGHTBLACK_EX, Fore.RESET)):
                url = image["link"]
                filename = str(i).zfill(3) + "_" + url.split("/")[-1]
                save_path = os.path.join(output_dir, filename)
                try:
                    if not os.path.exists(output_dir):
                        os.makedirs(output_dir)
                    urllib.request.urlretrieve(url, save_path)
                except Exception as e:
                    self.print_formatted_error(e)

    def is_imgur_image(self, url):
        return "imgur.com" in url

    def download_imgur_image(self, submission, output_dir):
        # Other imgur content, e.g., .gifv, '.mp4', '.jpg', etc.
        url_leaf = submission.url.split("/")[-1]
        if "." in url_leaf:
            image_id = url_leaf.split(".")[0]
        else:
            image_id = url_leaf

        try:
            data = self.get_imgur_image_meta(image_id)
            url = data["link"]
            image_type = data["type"]
            if "video/" in image_type:
                self.logger.spam(
                    self.indent_2 + "This is an imgur link to a video file")
                image_type = image_type.split("video/")[-1]
            elif "image/" in image_type:
                self.logger.spam(
                    self.indent_2 + "This is an imgur link to an image file")
                image_type = image_type.split("image/")[-1]

            filename = image_id + "." + image_type
            save_path = os.path.join(output_dir, filename)

            urllib.request.urlretrieve(url, save_path)
        except Exception as e:
            self.print_formatted_error(e)

    def download_comments(self, submission, output_dir, comment_limit):
        # Save comments - Breath first unwrap of comment forest
        comments_list = []
        with open(os.path.join(output_dir, 'comments.json'), 'w') as file:
            submission.comments.replace_more(limit=comment_limit)
            limited_comments = submission.comments.list()
            if not len(limited_comments):
                # No comments
                self.logger.spam(self.indent_2 + "No comments found")
                return

            for comment in tqdm(limited_comments, total=len(limited_comments), bar_format='%s%s{l_bar}{bar:20}{r_bar}%s' % (self.indent_2, Fore.WHITE + Fore.LIGHTBLACK_EX, Fore.RESET)):
                comment_dict = {}
                try:
                    if comment.author:
                        comment_dict["author"] = comment.author.name
                    else:
                        comment_dict["author"] = None
                    comment_dict["body"] = comment.body
                    comment_dict["created_utc"] = int(comment.created_utc)
                    comment_dict["distinguished"] = comment.distinguished
                    comment_dict["downs"] = comment.downs
                    comment_dict["edited"] = comment.edited
                    comment_dict["id"] = comment.id
                    comment_dict["is_submitter"] = comment.is_submitter
                    comment_dict["link_id"] = comment.link_id
                    comment_dict["parent_id"] = comment.parent_id
                    comment_dict["permalink"] = comment.permalink
                    comment_dict["score"] = comment.score
                    comment_dict["stickied"] = comment.stickied
                    comment_dict["subreddit_name_prefixed"] = comment.subreddit_name_prefixed
                    comment_dict["subreddit_id"] = comment.subreddit_id
                    comment_dict["total_awards_received"] = comment.total_awards_received
                    comment_dict["ups"] = comment.ups
                except Exception as e:
                    self.print_formatted_error(e)
                comments_list.append(comment_dict)
            file.write(json.dumps(comments_list, indent=2))

    def is_self_post(self, submission):
        return submission.is_self

    def download_submission_meta(self, submission, submission_dir):
        submission_dict = {}
        if submission.author:
            submission_dict["author"] = submission.author.name
        else:
            submission_dict["author"] = None
        submission_dict["created_utc"] = int(submission.created_utc)
        submission_dict["distinguished"] = submission.distinguished
        submission_dict["downs"] = submission.downs
        submission_dict["edited"] = submission.edited
        submission_dict["id"] = submission.id
        submission_dict["link_flair_text"] = submission.link_flair_text
        submission_dict["locked"] = submission.locked
        submission_dict["num_comments"] = submission.num_comments
        submission_dict["num_crossposts"] = submission.num_crossposts
        submission_dict["permalink"] = submission.permalink
        submission_dict["selftext"] = submission.selftext
        submission_dict["selftext"] = submission.selftext
        submission_dict["selftext_html"] = submission.selftext_html
        submission_dict["send_replies"] = submission.send_replies
        submission_dict["spoiler"] = submission.spoiler
        submission_dict["stickied"] = submission.stickied
        submission_dict["subreddit_name_prefixed"] = submission.subreddit_name_prefixed
        submission_dict["subreddit_id"] = submission.subreddit_id
        submission_dict["subreddit_subscribers"] = submission.subreddit_subscribers
        submission_dict["subreddit_type"] = submission.subreddit_type
        submission_dict["title"] = submission.title
        submission_dict["total_awards_received"] = submission.total_awards_received
        submission_dict["ups"] = submission.ups
        submission_dict["upvote_ratio"] = submission.upvote_ratio
        submission_dict["url"] = submission.url

        with open(os.path.join(submission_dir, "submission.json"), 'w') as file:
            file.write(json.dumps(submission_dict, indent=2))


import argparse
from saveddit.subreddit_downloader import SubredditDownloader
from saveddit.user_downloader import UserDownloader
import sys


def asciiart():
    return r'''                                .___  .___.__  __
   ___________ ___  __ ____   __| _/__| _/|__|/  |_
  /  ___/\__  \\  \/ // __ \ / __ |/ __ | |  \   __\
  \___ \  / __ \\   /\  ___// /_/ / /_/ | |  ||  |
 /____  >(____  /\_/  \___  >____ \____ | |__||__|
      \/      \/          \/     \/    \/

 Downloader for Reddit
 version : v2.0.1
 URL     : https://github.com/p-ranav/saveddit
'''


def check_positive(value):
    ivalue = int(value)
    if ivalue <= 0:
        raise argparse.ArgumentTypeError(
            "%s is an invalid positive int value" % value)
    return ivalue

class UniqueAppendAction(argparse.Action):
    '''
    Class used to discard duplicates in list arguments
    https://stackoverflow.com/questions/9376670/python-argparse-force-a-list-item-to-be-unique
    '''
    def __call__(self, parser, namespace, values, option_string=None):
        unique_values = set(values)
        setattr(namespace, self.dest, unique_values)

def main():
    print(asciiart())
    argv = sys.argv[1:]

    parser = argparse.ArgumentParser(prog="saveddit")
    subparsers = parser.add_subparsers(dest="subparser_name")

    subreddit_parser = subparsers.add_parser('subreddit')
    subreddit_parser.add_argument('subreddits',
                        metavar='subreddits',
                        nargs='+',
                        action=UniqueAppendAction,
                        help='Names of subreddits to download, e.g., AskReddit')
    subreddit_parser.add_argument('-f',
                        metavar='categories',
                        default=SubredditDownloader.DEFAULT_CATEGORIES,
                        nargs='+',
                        action=UniqueAppendAction,
                        help='Categories of posts to download (default: %(default)s)')
    subreddit_parser.add_argument('-l',
                        default=SubredditDownloader.DEFAULT_POST_LIMIT,
                        metavar='post_limit',
                        type=check_positive,
                        help='Limit the number of submissions downloaded in each category (default: %(default)s, i.e., all submissions)')
    subreddit_parser.add_argument('--skip-comments',
                        default=False,
                        action='store_true',
                        help='When true, saveddit will not save comments to a comments.json file')
    subreddit_parser.add_argument('--skip-meta',
                        default=False,
                        action='store_true',
                        help='When true, saveddit will not save meta to a submission.json file on submissions')
    subreddit_parser.add_argument('--skip-videos',
                        default=False,
                        action='store_true',
                        help='When true, saveddit will not download videos (e.g., gfycat, redgifs, youtube, v.redd.it links)')
    subreddit_parser.add_argument('-o',
                        required=True,
                        type=str,
                        metavar='output_path',
                        help='Directory where saveddit will save downloaded content'
                        )

    user_parser = subparsers.add_parser('user')
    user_parser.add_argument('users',
                        metavar='users',
                        nargs='+',
                        help='Names of users to download, e.g., Poem_for_your_sprog')


    user_subparsers = user_parser.add_subparsers(dest="user_subparser_name")
    user_subparsers.required = True

    # user.saved subparser
    saved_parser = user_subparsers.add_parser('saved')
    saved_parser.add_argument('--skip-meta',
                        default=False,
                        action='store_true',
                        help='When true, saveddit will not save meta to a submission.json file on submissions')
    saved_parser.add_argument('--skip-comments',
                        default=False,
                        action='store_true',
                        help='When true, saveddit will not save comments to a comments.json file')
    saved_parser.add_argument('--skip-videos',
                        default=False,
                        action='store_true',
                        help='When true, saveddit will not download videos (e.g., gfycat, redgifs, youtube, v.redd.it links)')
    saved_parser.add_argument('-l',
                        default=UserDownloader.DEFAULT_POST_LIMIT,
                        metavar='post_limit',
                        type=check_positive,
                        help='Limit the number of saved submissions downloaded (default: %(default)s, i.e., all submissions)')
    saved_parser.add_argument('-o',
                        required=True,
                        type=str,
                        metavar='output_path',
                        help='Directory where saveddit will save downloaded content'
                        )

    # user.gilded subparser
    gilded_parser = user_subparsers.add_parser('gilded')
    gilded_parser.add_argument('--skip-meta',
                        default=False,
                        action='store_true',
                        help='When true, saveddit will not save meta to a submission.json file on submissions')
    gilded_parser.add_argument('--skip-comments',
                        default=False,
                        action='store_true',
                        help='When true, saveddit will not save comments to a comments.json file')
    gilded_parser.add_argument('--skip-videos',
                        default=False,
                        action='store_true',
                        help='When true, saveddit will not download videos (e.g., gfycat, redgifs, youtube, v.redd.it links)')
    gilded_parser.add_argument('-l',
                        default=UserDownloader.DEFAULT_POST_LIMIT,
                        metavar='post_limit',
                        type=check_positive,
                        help='Limit the number of saved submissions downloaded (default: %(default)s, i.e., all submissions)')
    gilded_parser.add_argument('-o',
                        required=True,
                        type=str,
                        metavar='output_path',
                        help='Directory where saveddit will save downloaded content'
                        )

    # user.submitted subparser
    submitted_parser = user_subparsers.add_parser('submitted')
    submitted_parser.add_argument('-s',
                        metavar='sort',
                        default=UserDownloader.DEFAULT_SORT,
                        choices=UserDownloader.DEFAULT_SORT_OPTIONS,
                        help='Download submissions sorted by this <sort> option (default: %(default)s, choices: [%(choices)s])')
    submitted_parser.add_argument('--skip-comments',
                        default=False,
                        action='store_true',
                        help='When true, saveddit will not save comments to a comments.json file for the submissions')
    submitted_parser.add_argument('--skip-meta',
                        default=False,
                        action='store_true',
                        help='When true, saveddit will not save meta to a submission.json file on submissions')
    submitted_parser.add_argument('--skip-videos',
                        default=False,
                        action='store_true',
                        help='When true, saveddit will not download videos (e.g., gfycat, redgifs, youtube, v.redd.it links)')
    submitted_parser.add_argument('-l',
                        default=UserDownloader.DEFAULT_POST_LIMIT,
                        metavar='post_limit',
                        type=check_positive,
                        help='Limit the number of submissions downloaded (default: %(default)s, i.e., all submissions)')
    submitted_parser.add_argument('-o',
                        required=True,
                        type=str,
                        metavar='output_path',
                        help='Directory where saveddit will save downloaded posts'
                        )

    # user.upvoted subparser
    upvoted_parser = user_subparsers.add_parser('upvoted')
    upvoted_parser.add_argument('--skip-comments',
                        default=False,
                        action='store_true',
                        help='When true, saveddit will not save comments to a comments.json file for the upvoted submissions')
    upvoted_parser.add_argument('--skip-meta',
                        default=False,
                        action='store_true',
                        help='When true, saveddit will not save meta to a submission.json file on upvoted submissions')
    upvoted_parser.add_argument('--skip-videos',
                        default=False,
                        action='store_true',
                        help='When true, saveddit will not download videos (e.g., gfycat, redgifs, youtube, v.redd.it links)')
    upvoted_parser.add_argument('-l',
                        default=UserDownloader.DEFAULT_POST_LIMIT,
                        metavar='post_limit',
                        type=check_positive,
                        help='Limit the number of submissions downloaded (default: %(default)s, i.e., all submissions)')
    upvoted_parser.add_argument('-o',
                        required=True,
                        type=str,
                        metavar='output_path',
                        help='Directory where saveddit will save downloaded posts'
                        )

    # user.comments subparser
    comments_parser = user_subparsers.add_parser('comments')
    comments_parser.add_argument('-s',
                        metavar='sort',
                        default=UserDownloader.DEFAULT_SORT,
                        choices=UserDownloader.DEFAULT_SORT_OPTIONS,
                        help='Download comments sorted by this <sort> option (default: %(default)s, choices: [%(choices)s])')
    comments_parser.add_argument('-l',
                        default=UserDownloader.DEFAULT_COMMENT_LIMIT,
                        metavar='post_limit',
                        type=check_positive,
                        help='Limit the number of comments downloaded (default: %(default)s, i.e., all comments)')
    comments_parser.add_argument('-o',
                        required=True,
                        type=str,
                        metavar='output_path',
                        help='Directory where saveddit will save downloaded comments'
                        )

    args = parser.parse_args(argv)

    if args.subparser_name == "subreddit":
        for subreddit in args.subreddits:
            downloader = SubredditDownloader(subreddit)
            downloader.download(args.o,
                                categories=args.f, post_limit=args.l, skip_videos=args.skip_videos, skip_meta=args.skip_meta, skip_comments=args.skip_comments)
    elif args.subparser_name == "user":
        downloader = UserDownloader()
        if args.user_subparser_name == "comments":
            downloader.download_comments(args)
        elif args.user_subparser_name == "submitted":
            downloader.download_submitted(args)
        elif args.user_subparser_name == "saved":
            downloader.download_saved(args)
        elif args.user_subparser_name == "upvoted":
            downloader.download_upvoted(args)
        elif args.user_subparser_name == "gilded":
            downloader.download_gilded(args)

if __name__ == "__main__":
    main()

# saveddit user p_ranav saved --skip-comments --skip-videos -o ~/Desktop
# saveddit user p_ranav submitted --sort hot new controversial top --skip-videos -o ~/Desktop
# saveddit user p_ranav comments --sort hot new controversial top -o ~/Desktop


import coloredlogs
from colorama import Fore, Style
from datetime import datetime, timezone
import logging
import verboselogs
import getpass
import json
import os
import praw
from pprint import pprint
import re
from saveddit.submission_downloader import SubmissionDownloader
from saveddit.subreddit_downloader import SubredditDownloader
import sys
from tqdm import tqdm


class UserDownloader():
    config = SubredditDownloader.config

    REDDIT_CLIENT_ID = config['reddit_client_id']
    REDDIT_CLIENT_SECRET = config['reddit_client_secret']
    try:
        REDDIT_USERNAME = config['reddit_username']
    except Exception as e:
        print(Fore.RED + 'ERROR: Failed to find value for "reddit_username" in user_config.yaml')
        print("Create an entry in user_config.yaml:")
        print("  'reddit_username': <YOUR_REDDIT_USERNAME>")
        print(Style.RESET_ALL, end="")
        print('Exiting now')
        exit()

    REDDIT_PASSWORD = None
    if len(REDDIT_USERNAME):
        if sys.stdin.isatty():
            print("Username: " + REDDIT_USERNAME)
            REDDIT_PASSWORD = getpass.getpass("Password: ")
        else:
            # echo "foobar" > password
            # saveddit user .... < password
            REDDIT_PASSWORD = sys.stdin.readline().rstrip()

    IMGUR_CLIENT_ID = config['imgur_client_id']
    DEFAULT_CATEGORIES = ["comments", "submitted", "saved"]
    DEFAULT_SORT = "hot"
    DEFAULT_SORT_OPTIONS = ["hot", "new", "top", "controversial"]
    DEFAULT_POST_LIMIT = None
    DEFAULT_COMMENT_LIMIT = None

    def __init__(self):
        self.logger = verboselogs.VerboseLogger(__name__)
        level_styles = {
            'critical': {'bold': True, 'color': 'red'},
            'debug': {'color': 'green'},
            'error': {'color': 'red'},
            'info': {'color': 'white'},
            'notice': {'color': 'magenta'},
            'spam': {'color': 'white', 'faint': True},
            'success': {'bold': True, 'color': 'green'},
            'verbose': {'color': 'blue'},
            'warning': {'color': 'yellow'}
        }
        coloredlogs.install(level='SPAM', logger=self.logger,
                            fmt='%(message)s', level_styles=level_styles)

        if not len(UserDownloader.REDDIT_USERNAME):
            self.logger.error("`reddit_username` in user_config.yaml is empty")
            self.logger.error("If you plan on using the user API of saveddit, then add your username to user_config.yaml")
            print("Exiting now")
            exit()

        if not len(UserDownloader.REDDIT_PASSWORD):
            if sys.stdin.isatty():
                print("Username: " + REDDIT_USERNAME)
                REDDIT_PASSWORD = getpass.getpass("Password: ")
            else:
                # echo "foobar" > password
                # saveddit user .... < password
                REDDIT_PASSWORD = sys.stdin.readline().rstrip()

        self.reddit = praw.Reddit(
            client_id=UserDownloader.REDDIT_CLIENT_ID,
            client_secret=UserDownloader.REDDIT_CLIENT_SECRET,
            user_agent="saveddit (by /u/p_ranav)",
            username=UserDownloader.REDDIT_USERNAME,
            password=UserDownloader.REDDIT_PASSWORD
        )

    def download_comments(self, args):
        output_path = args.o

        for username in args.users:
            user = self.reddit.redditor(name=username)

            self.logger.notice("Downloading from /u/" + username + "/comments")

            root_dir = os.path.join(os.path.join(os.path.join(
                output_path, "www.reddit.com"), "u"), username)

            try:
                sort = args.s
                limit = args.l

                comments_dir = os.path.join(root_dir, "comments")
                if not os.path.exists(comments_dir):
                    os.makedirs(comments_dir)

                self.logger.verbose("Downloading comments sorted by " + sort)
                category_function = getattr(user.comments, sort)

                category_dir = os.path.join(comments_dir, sort)

                if category_function:
                    if not os.path.exists(category_dir):
                        os.makedirs(category_dir)
                    for i, comment in enumerate(category_function(limit=limit)):
                        prefix_str = '#' + str(i).zfill(3) + ' '
                        self.indent_1 = ' ' * len(prefix_str) + "* "
                        self.indent_2 = ' ' * len(self.indent_1) + "- "

                        comment_body = comment.body
                        comment_body = comment_body[0:32]
                        comment_body = re.sub(r'\W+', '_', comment_body)
                        comment_filename = str(i).zfill(3) + "_Comment_" + \
                            comment_body + "..." + ".json"
                        self.logger.spam(self.indent_1 + comment.id + ' - "' + comment.body[0:64].replace("\n", "").replace("\r", "")  + '..."')

                        with open(os.path.join(category_dir, comment_filename), 'w') as file:
                            comment_dict = {}
                            try:
                                if comment.author:
                                    comment_dict["author"] = comment.author.name
                                else:
                                    comment_dict["author"] = None
                                comment_dict["body"] = comment.body
                                comment_dict["created_utc"] = int(comment.created_utc)
                                comment_dict["distinguished"] = comment.distinguished
                                comment_dict["downs"] = comment.downs
                                comment_dict["edited"] = comment.edited
                                comment_dict["id"] = comment.id
                                comment_dict["is_submitter"] = comment.is_submitter
                                comment_dict["link_id"] = comment.link_id
                                comment_dict["parent_id"] = comment.parent_id
                                comment_dict["permalink"] = comment.permalink
                                comment_dict["score"] = comment.score
                                comment_dict["stickied"] = comment.stickied
                                comment_dict["subreddit_name_prefixed"] = comment.subreddit_name_prefixed
                                comment_dict["subreddit_id"] = comment.subreddit_id
                                comment_dict["total_awards_received"] = comment.total_awards_received
                                comment_dict["ups"] = comment.ups
                                file.write(json.dumps(comment_dict, indent=2))
                            except Exception as e:
                                self.print_formatted_error(e)
            except Exception as e:
                self.logger.error("Unable to download comments for user `" + username + "` - " + str(e))

    def download_submitted(self, args):
        output_path = args.o

        for username in args.users:
            user = self.reddit.redditor(name=username)

            self.logger.notice("Downloading from /u/" + username + "/submitted")

            root_dir = os.path.join(os.path.join(os.path.join(
                output_path, "www.reddit.com"), "u"), username)

            try:
                post_limit = args.l
                sort = args.s
                skip_meta = args.skip_meta
                skip_videos = args.skip_videos
                skip_comments = args.skip_comments
                comment_limit = 0 # top-level comments ONLY

                submitted_dir = os.path.join(root_dir, "submitted")
                if not os.path.exists(submitted_dir):
                    os.makedirs(submitted_dir)

                self.logger.verbose("Downloading submissions sorted by " + sort)
                category_function = getattr(user.submissions, sort)

                category_dir = os.path.join(submitted_dir, sort)

                if category_function:
                    for i, s in enumerate(category_function(limit=post_limit)):
                        prefix_str = '#' + str(i).zfill(3) + ' '
                        self.indent_1 = ' ' * len(prefix_str) + "* "
                        self.indent_2 = ' ' * len(self.indent_1) + "- "
                        SubmissionDownloader(s, i, self.logger, category_dir, skip_videos, skip_meta, skip_comments, comment_limit,
                                                {'imgur_client_id': UserDownloader.IMGUR_CLIENT_ID})
            except Exception as e:
                self.logger.error("Unable to download submitted posts for user `" + username + "` - " + str(e))

    def download_upvoted(self, args):
        output_path = args.o

        for username in args.users:
            user = self.reddit.redditor(name=username)

            self.logger.notice("Downloading from /u/" + username + "/upvoted")

            root_dir = os.path.join(os.path.join(os.path.join(
                output_path, "www.reddit.com"), "u"), username)

            try:
                post_limit = args.l
                skip_meta = args.skip_meta
                skip_videos = args.skip_videos
                skip_comments = args.skip_comments
                comment_limit = 0 # top-level comments ONLY

                upvoted_dir = os.path.join(root_dir, "upvoted")
                if not os.path.exists(upvoted_dir):
                    os.makedirs(upvoted_dir)

                for i, s in enumerate(user.upvoted(limit=post_limit)):
                    prefix_str = '#' + str(i).zfill(3) + ' '
                    self.indent_1 = ' ' * len(prefix_str) + "* "
                    self.indent_2 = ' ' * len(self.indent_1) + "- "
                    SubmissionDownloader(s, i, self.logger, upvoted_dir, skip_videos, skip_meta, skip_comments, comment_limit,
                                            {'imgur_client_id': UserDownloader.IMGUR_CLIENT_ID})
            except Exception as e:
                self.logger.error("Unable to download upvoted posts for user `" + username + "` - " + str(e))

    def download_saved(self, args):
        output_path = args.o

        for username in args.users:
            user = self.reddit.redditor(name=username)

            self.logger.notice("Downloading from /u/" + username + "/saved")

            root_dir = os.path.join(os.path.join(os.path.join(
                output_path, "www.reddit.com"), "u"), username)

            try:
                post_limit = args.l
                skip_meta = args.skip_meta
                skip_videos = args.skip_videos
                skip_comments = args.skip_comments
                comment_limit = 0 # top-level comments ONLY

                saved_dir = os.path.join(root_dir, "saved")
                if not os.path.exists(saved_dir):
                    os.makedirs(saved_dir)

                for i, s in enumerate(user.saved(limit=post_limit)):
                    prefix_str = '#' + str(i).zfill(3) + ' '
                    self.indent_1 = ' ' * len(prefix_str) + "* "
                    self.indent_2 = ' ' * len(self.indent_1) + "- "
                    if isinstance(s, praw.models.Comment) and not skip_comments:
                        self.logger.verbose(
                            prefix_str + "Comment `" + str(s.id) + "` by " + str(s.author) + " \"" + s.body[0:32].replace("\n", "").replace("\r", "") + "...\"")

                        comment_body = s.body
                        comment_body = comment_body[0:32]
                        comment_body = re.sub(r'\W+', '_', comment_body)
                        post_dir = str(i).zfill(3) + "_Comment_" + \
                            comment_body + "..."
                        submission_dir = os.path.join(saved_dir, post_dir)
                        self.download_saved_comment(s, submission_dir)
                    elif isinstance(s, praw.models.Comment):
                        self.logger.verbose(
                            prefix_str + "Comment `" + str(s.id) + "` by " + str(s.author))
                        self.logger.spam(self.indent_2 + "Skipping comment")
                    elif isinstance(s, praw.models.Submission):
                        SubmissionDownloader(s, i, self.logger, saved_dir, skip_videos, skip_meta, skip_comments, comment_limit,
                                            {'imgur_client_id': UserDownloader.IMGUR_CLIENT_ID})
                    else:
                        pass
            except Exception as e:
                self.logger.error("Unable to download saved for user `" + username + "` - " + str(e))

    def download_gilded(self, args):
        output_path = args.o

        for username in args.users:
            user = self.reddit.redditor(name=username)

            self.logger.notice("Downloading from /u/" + username + "/gilded")

            root_dir = os.path.join(os.path.join(os.path.join(
                output_path, "www.reddit.com"), "u"), username)

            try:
                post_limit = args.l
                skip_meta = args.skip_meta
                skip_videos = args.skip_videos
                skip_comments = args.skip_comments
                comment_limit = 0 # top-level comments ONLY

                saved_dir = os.path.join(root_dir, "gilded")
                if not os.path.exists(saved_dir):
                    os.makedirs(saved_dir)

                for i, s in enumerate(user.gilded(limit=post_limit)):
                    prefix_str = '#' + str(i).zfill(3) + ' '
                    self.indent_1 = ' ' * len(prefix_str) + "* "
                    self.indent_2 = ' ' * len(self.indent_1) + "- "
                    if isinstance(s, praw.models.Comment) and not skip_comments:
                        self.logger.verbose(
                            prefix_str + "Comment `" + str(s.id) + "` by " + str(s.author) + " \"" + s.body[0:32].replace("\n", "").replace("\r", "") + "...\"")

                        comment_body = s.body
                        comment_body = comment_body[0:32]
                        comment_body = re.sub(r'\W+', '_', comment_body)
                        post_dir = str(i).zfill(3) + "_Comment_" + \
                            comment_body + "..."
                        submission_dir = os.path.join(saved_dir, post_dir)
                        self.download_saved_comment(s, submission_dir)
                    elif isinstance(s, praw.models.Comment):
                        self.logger.verbose(
                            prefix_str + "Comment `" + str(s.id) + "` by " + str(s.author))
                        self.logger.spam(self.indent_2 + "Skipping comment")
                    elif isinstance(s, praw.models.Submission):
                        SubmissionDownloader(s, i, self.logger, saved_dir, skip_videos, skip_meta, skip_comments, comment_limit,
                                            {'imgur_client_id': UserDownloader.IMGUR_CLIENT_ID})
                    else:
                        pass
            except Exception as e:
                self.logger.error("Unable to download gilded for user `" + username + "` - " + str(e))

    def print_formatted_error(self, e):
        for line in str(e).split("\n"):
            self.logger.error(self.indent_2 + line)

    def download_saved_comment(self, comment, output_dir):
        if not os.path.exists(output_dir):
            os.makedirs(output_dir)
        self.logger.spam(
            self.indent_2 + "Saving comment.json to " + output_dir)
        with open(os.path.join(output_dir, 'comments.json'), 'w') as file:
            comment_dict = {}
            try:
                if comment.author:
                    comment_dict["author"] = comment.author.name
                else:
                    comment_dict["author"] = None
                comment_dict["body"] = comment.body
                comment_dict["created_utc"] = int(comment.created_utc)
                comment_dict["distinguished"] = comment.distinguished
                comment_dict["downs"] = comment.downs
                comment_dict["edited"] = comment.edited
                comment_dict["id"] = comment.id
                comment_dict["is_submitter"] = comment.is_submitter
                comment_dict["link_id"] = comment.link_id
                comment_dict["parent_id"] = comment.parent_id
                comment_dict["permalink"] = comment.permalink
                comment_dict["score"] = comment.score
                comment_dict["stickied"] = comment.stickied
                comment_dict["subreddit_name_prefixed"] = comment.subreddit_name_prefixed
                comment_dict["subreddit_id"] = comment.subreddit_id
                comment_dict["total_awards_received"] = comment.total_awards_received
                comment_dict["ups"] = comment.ups
                file.write(json.dumps(comment_dict, indent=2))
                self.logger.spam(
                    self.indent_2 + "Successfully saved comment.json")
            except Exception as e:
                self.print_formatted_error(e)